import gql from "graphql-tag";
import { useLazyQuery } from "@vue/apollo-composable";
import { computed, nextTick, ref } from "vue";
const prepareQuery = (query) => {
    return query
        .split("@pageInfo")
        .join("pageInfo { hasNextPage hasPreviousPage startCursor endCursor }");
};
export const getValueByPath = (object, path) => {
    const pathItems = path.split(".");
    return pathItems.reduce((acc, key, currentIndex) => {
        if (acc[key] == null && currentIndex > 0) {
            throw new Error(`Unable to resolve edges. Key=${key} not found in the response`);
        }
        return acc[key];
    }, object);
};
export const useQuery = (name, query, options) => {
    const doc = `query ${name} ${prepareQuery(query)}`;
    const document = gql `
    ${doc}
  `;
    const enabled = ref(options?.enabled ?? true);
    const config = () => ({
        notifyOnNetworkStatusChange: true,
        fetchPolicy: options?.cacheOrLoad === true ? "cache-first" : "cache-and-network",
        nextFetchPolicy: "cache-and-network",
        enabled: enabled.value,
    });
    const { result, loading, load, refetch, networkStatus, fetchMore, variables, error, } = useLazyQuery(document, options?.variables, 
    // @ts-ignore
    config);
    const loadQuery = (variables) => {
        if (networkStatus.value === 7 || networkStatus.value === 8) {
            return refetch(variables);
        }
        else {
            return load(undefined, variables);
        }
    };
    if (options?.lazy !== true) {
        nextTick(() => loadQuery()).catch(console.error);
    }
    return {
        isLoading: computed(() => loading.value),
        isLoadingMore: computed(() => networkStatus.value === 3),
        data: computed(() => {
            if (result.value != null && options?.select != null) {
                return getValueByPath(result.value, options.select);
            }
            return result.value;
        }),
        variables,
        error,
        load: loadQuery,
        loadMore(variables, updateQuery) {
            return fetchMore({ variables, updateQuery });
        },
        status: networkStatus,
        disable() {
            enabled.value = false;
        },
        enable() {
            enabled.value = true;
        },
    };
};
